precision highp float;

uniform sampler2D tex_d;
uniform sampler2D tex_n;
uniform sampler2D tex_s;
uniform sampler2D tex_g;
uniform sampler2D tex_t;

uniform vec3 u_v3LightDiffuse;
uniform vec3 u_v3LightAmbient;

uniform vec3 u_v3Specular;
uniform float u_fSpecMultiplier;


varying vec3 v_v3Normal;


varying highp vec3 v_v3Light_Transformed;

varying vec2 v_v2TexCoord;
varying vec2 v_v2TexCoord2;

#ifdef WATER_LOW_RES_WATER
varying vec4 v_v4TexCoordDepth;
uniform sampler2D tex_depth;
uniform sampler2D tex_background;
#endif

varying vec3 v_v3ViewDir;

varying vec4 v_v4Colour;

void main() 
{
	vec4 v4TextureD = texture2D(tex_d, v_v2TexCoord);

    
#ifdef TRANSLUCENT
    gl_FragColor = v4TextureD * v_v4Colour;
    
#else // TRANSLUCENT
    
    vec4 v4TextureS = texture2D(tex_s, v_v2TexCoord);
    vec4 v4TextureT;
    
    highp vec3 v3Normal;
    
#ifdef TANGENT_NORMAL_MAP
	
	v3Normal = vec3(texture2D(tex_n,v_v2TexCoord.st));
	v3Normal -= 0.5;
	v3Normal *= 1.0;

#else // Use vertex normals
	v3Normal = v_v3Normal;
#endif

	highp float _NdotL = dot(v_v3Light_Transformed, v3Normal);
	highp float NdotL = _NdotL * 0.5 + 0.5;
    vec2 v2Transmission = vec2(NdotL,0.5);
    v4TextureT =  texture2D(tex_t, v2Transmission);
    vec3 v3LightDiffuse = u_v3LightDiffuse * 1.40;
    
    vec3 v3CombinedLights = vec3(v4TextureT) * v3LightDiffuse;
	

	float fSpec = 0.0;
	vec3 v3Reflection;
#if 1 // Specular calculation
	float lightIncident = _NdotL;
	
	float fGlancing = (dot(v3Normal, v_v3ViewDir));
	float fBaseSpecular;
	//if (lightIncident > 0.0)
	{
		v3Reflection = normalize(reflect(v_v3Light_Transformed, v3Normal));

		fBaseSpecular = dot(v3Reflection, v_v3ViewDir);
#ifdef SURFBOARD
		float fSpecHack = 0.05;
#else //  SURFBOARD

		float fSpecHack = u_fSpecMultiplier;
#endif //  SURFBOARD


	}
#endif // Specular calculation

	gl_FragColor = vec4(v3CombinedLights, 1.0)* v4TextureD;

	float fGlancingVal = clamp(fGlancing, 0.0, 1.0);
	float fSpecVal = clamp(fBaseSpecular, 0.0, 1.0);

	fSpecVal *= fGlancingVal*u_fSpecMultiplier*25.0;

	gl_FragColor.rgb += v4TextureS.rgb*fSpecVal;

#ifndef SURFBOARD
    gl_FragColor.a = v4TextureD.a;
#endif
	
	
	gl_FragColor = gl_FragColor * v_v4Colour;
    /*
    gl_FragColor = vec4( NdotL, NdotL, NdotL, 1.0);
    
    if(NdotL < -0.5)
        gl_FragColor = vec4( 1.0, 0.0, 0.0, 1.0);
    if(NdotL > 1.5)
        gl_FragColor = vec4( 0.0, 1.0, 0.0, 1.0);
    */
     /*
    vec3 outNormal = (v3Normal + vec3(1.0,1.0,1.0) ) * 0.5;
    
    
    gl_FragColor = vec4(outNormal, 1.0);
*/

#endif // TRANSLUCENT

#ifdef WATER_LOW_RES_WATER

		vec3 v3BG = texture2DProj(tex_background, v_v4TexCoordDepth).rgb;
	float fDiff = gl_FragCoord.z - texture2DProj(tex_depth, v_v4TexCoordDepth).r;
	fDiff += (0.2 /  4000.0) * (v3BG.r - 0.1);


	//gl_FragColor.rgb += texture2DProj(tex_depth, v_v4TexCoordDepth).rgb * 0.5 - gl_FragColor.rgb * 0.5; 
	if (fDiff > 0.0)
	{
		fDiff *= 4000.0;
		gl_FragColor.rgb *= v3BG * v3BG;
		gl_FragColor.rgb += v3BG * 0.5;
		if (fDiff > 1.0)
			fDiff = 1.0;

		gl_FragColor.rgb += (v3BG - gl_FragColor.rgb) * fDiff; 
	}

	//gl_FragColor.rgb = vec3( texture2DProj(tex_depth, v_v4TexCoordDepth).x * 0.1);

	//gl_FragColor.rgb += texture2DProj(tex_depth, v_v4TexCoordDepth).rgb * 0.5 - gl_FragColor.rgb * 0.5; 
#endif


}
